/**
 * Introduction
 *
 * @title ROV - Easy applications from C
 *
 * @copyright Jason Tribbeck / 7th ARM 1994-2001
 *
 * @authors Jason Tribbeck
 *
 * @file menu.h
 *
 * @type core
 *
 * @description
 *
 * The menu core system provides all the services for creating and
 * displaying menus.
 */

#ifndef __rov_menu
#define __rov_menu


#ifdef __cplusplus
extern "C"
{
#endif

#ifndef __rov_main
#include "main.h"
#endif

#ifndef __rov_canvas
#include "canvas.h"
#endif

/**
 * Structure of an item in a path of menus (for decoding menu
 * clicks)
 * @param i the index in the menu (-1 for no more).
 * @param m the menu holding the index (0 for no more).
 */
typedef struct {
  int i;
  menu m;
} menu_list;

/**
 * Definition of a menu handler.
 * @param m the menu that was selected.
 * @param index the index into the menu.
 * @param menu_path the path of opened menus from the top to the selected menu.
 * @param index_path the index of items underneath the last attached menu (used
 * for sub-menu creation).
 * @note If submenus are not being used, then it is normally easier to
 * use a simplified interface where the last two parameters are ignored.
 */
typedef void (*_rov_menuh)(menu m,
			   int index,
			   menu_list menu_path[],
			   int index_path[]);

/**
 * Definition of a sub-menu handler.
 * @param m the menu that was selected.
 * @param index the index into the menu.
 * @param menu_path the path of opened menus from the top to the selected menu.
 */
typedef void (* _rov_submenuh)(menu m,
			       int index,
			       menu_list menu_path[]);

#ifdef OLDMENU_STYLE
menu menu_create(char * , int, _rov_menuh, _rov_submenuh); 	/* Create menu (title, max items,
							 choose_handler, submenu_handler)	 */
#else
/**
 * Create a menu.
 * @param title the menu title.
 * @param max_items the maximum number of items in this menu.
 * @param ... two parameters that are the functions for handling menu choices,
 * and sub-menu creation.
 * @return the created menu.
 * @note If a handler is not required, then NULL can be passed.
 */
menu menu_create(char *title,
		 int max_items,
		 ...);
#endif

/**
 * Initialise the menu system.
 * @note This must be done before menus are displayed.
 */
void menu_init(void);

/**
 * Add an item to a menu.
 * @param m the menu.
 * @param text the text of the menu item.
 * @note If the text is 12 characters or less in length, then the text
 * is copied into the menu structure itself. If it is more than 12
 * characters, then the pointer is used.
 * @see menu_addwriteable()
 * @see menu_changetext()
 */
void menu_add(menu m,
	      char *text);

/**
 * Zero the number of items in a menu.
 * @note This differs from RISC_OSLib's menu_clear() function in that
 * the ROV version does not free the memory.
 */
void menu_clear(menu m);

/**
 * Display a menu.
 * @param m the menu.
 * @note The position for this is taken from the mouse coordinates, with
 * the x coordinate subtracted by 48, in accordance with the RISC OS
 * Style Guide.
 */
void menu_display(menu m);

/**
 * Display a menu at a given location.
 * @param m the menu.
 * @param x the (screen) x coordinate.
 * @param y the (screen) y coorinate.
 */
void menu_displaylocation(menu m,
			  int x,
			  int y);

/**
 * Display a menu from an icon on the icon bar.
 * @param m the menu.
 * @note ROVLib calculates the correct y coordinate based on the number
 * of items in the menu, and any splitters in the menu.
 */
void menu_displayiconbar(menu m);

/**
 * Add a writeable menu item.
 * @param m the menu.
 * @param buffer the pointer to the buffer.
 * @param valid the validation string.
 * @param length the maximum string length.
 * @see menu_add()
 */
void menu_addwriteable(menu m,
		       char *buffer,
		       char *valid,
		       int length);

/**
 * Set the foreground colour of a menu item.
 * @param m the menu.
 * @param index the item index.
 * @param col the WIMP colour.
 * @note ROVLib will set the correct colour for you based on the Style Guide,
 * but this allows you to override it (for example, the colourmenu helper
 * does this).
 * @see colourmenu.h
 * @see menu_setback()
 */
void menu_setfore(menu m,
		  int index,
		  int col);

/**
 * Set the background colour of a menu item.
 * @param m the menu.
 * @param index the item index.
 * @param col the WIMP colour.
 * @note ROVLib will set the correct colour for you based on the Style Guide,
 * but this allows you to override it (for example, the colourmenu helper
 * does this).
 * @see colourmenu.h
 * @see menu_setback()
 */
void menu_setback(menu m,
		  int index,
		  int col);

/**
 * Fade/unfade a menu item.
 * @param m the menu.
 * @param index the item index.
 * @param fade non-zero fades the menu item, zero unfades the item.
 */
void menu_fade(menu m,
	       int index,
	       int fade);

/**
 * Tick/untick a menu item.
 * @param m the menu.
 * @param index the item index.
 * @param tick non-zero ticks the menu item, zero unticks the item.
 */
void menu_tick(menu m,
	       int index,
	       int tick);

/**
 * Attach a sub-menu to a menu.
 * @param parent the parent menu.
 * @param index the index.
 * @param child the child menu.
 */
void menu_submenu(menu parent,
		  int index,
		  menu child);

/**
 * Attach a canvas to a menu.
 * @param parent the parent menu.
 * @param index the index.
 * @param c the canvas.
 * @note This is typically used to put program information windows and
 * file information windows into a menu structure.
 * @note Clicking the canvas will not cause a call to the menu choose
 * handler.
 */
void menu_attachpane(menu parent,
		     int index,
		     canvas c);

/**
 * Change the text of a menu item.
 * @param m the menu.
 * @param index the index.
 * @param text the new text.
 * @see menu_add()
 */
void menu_changetext(menu m,
		     int index,
		     char *text);

/**
 * Split at a point.
 * @param m the menu.
 * @param index the index to split after.
 * @param split non-zero to split, zero to remove split.
 */
void menu_split(menu m,
		int index,
		int split);

/**
 * Display a sub-menu.
 * @param m the menu to display.
 * @note This is called in a sub-menu handler to display the
 * next level of menu.
 */
void menu_displaysubmenu(menu m);

/**
 * Display a canvas as a sub-menu.
 * @param c the canvas to display
 * @note This is called in a sub-menu handler to display a
 * canvas as the next level of menu,
 */
void menu_displaysubcanvas(canvas c);

/**
 * Display a canvas using menu methods.
 * @param c the canvas
 */
void menu_displaycanvas(canvas c);

/**
 * Make a menu item a sub menu with a warning.
 * @param m the menu.
 * @param index the index.
 * @note This will call the menu's sub menu warning handler.
 */
void menu_subwarning(menu m,
		     int index);

/**
 * Make a menu appear to the right of an icon.
 * @param m the menu.
 * @param ic the icon.
 */
void menu_displayicon(menu m,
		      icon ic);

/**
 * Make a canvas appear as a menu to the right of an icon.
 * @param c the canvas.
 * @param ic the ico.
 */
void menu_displaycanvasicon(canvas c,
			    icon ic);

/**
 * Remove all menus from the screen.
 */
void menu_remove(void);

/**
 * Set the title of a menu.
 * @param m the menu.
 * @param title the title.
 */
void menu_settitle(menu m,
		   char *title);

/**
 * Ensure the width of a menu is correct.
 * @param m the menu.
 */
void menu_checkwidth(menu m);

/**
 * A null menu.
 */
#define MENU_NULL ((menu)-1)


#ifdef __cplusplus
}
#endif

#endif
