/**
 * Introduction
 *
 * @title ROV - Easy applications from C
 *
 * @copyright Jason Tribbeck / 7th ARM 1994-2001
 *
 * @authors Jason Tribbeck
 *
 * @file toolbars.c
 */

#include <stdio.h>
#include <string.h>

#include "main.h"
#include "canvas.h"
#include "icons.h"
#include "draw.h"

#define TOOL_TOP       (0x00)
#define TOOL_RIGHT     (0x01)
#define TOOL_BOTTOM    (0x02)
#define TOOL_LEFT      (0x03)
/* A scrolling toolbar will scroll with the window - left / right will scroll vertically;
   top / bottom horizontally */
#define TOOL_STATIC    (0x00)
#define TOOL_SCROLL    (0x04)
/* A context - senstive toolbar basically changes its icons, depending on its function */
#define TOOL_NOCONTEXT (0x00)
#define TOOL_CONTEXT   (0x08)
/* Enable vertical scroll - bars */
#define TOOL_NOVSCROLL	(0x00)
#define TOOL_VSCROLL	(0x10)
/* Enable horizontal scroll - bars */
#define TOOL_NOHSCROLL	(0x00)
#define TOOL_HSCROLL	(0x20)

#define TOOL_SCROLLTOP    (TOOL_TOP|TOOL_SCROLL)
#define TOOL_SCROLLRIGHT  (TOOL_RIGHT|TOOL_SCROLL)
#define TOOL_SCROLLBOTTOM (TOOL_BOTTOM|TOOL_SCROLL)
#define TOOL_SCROLLLEFT   (TOOL_LEFT|TOOL_SCROLL)

#define TOOL_STATICTOP    (TOOL_TOP|TOOL_STATIC)
#define TOOL_STATICRIGHT  (TOOL_RIGHT|TOOL_STATIC)
#define TOOL_STATICBOTTOM (TOOL_BOTTOM|TOOL_STATIC)
#define TOOL_STATICLEFT   (TOOL_LEFT|TOOL_STATIC)

/* These are for icons - a click - icon will depress, and release when the click procedure has completed;
   toggle - icons will toggle on / off */
#define TOOL_CLICK        (0x00)
#define TOOL_TOGGLE	  (0x01)

typedef void(* tool_help)(void * , int);
typedef void(* tool_click)(void * , canvas_mousestr * , int);
typedef void(* tool_resize)(void * , int, int);
typedef void(* tool_closep)(void *);
typedef void(* tool_redrawp)(void * , int, int, int, int);

typedef struct toolicon {
  int x, y, flags;
  void * spr[4], * spra;
  /* canvas ca; */
  icon ic;
  struct toolicon * next;
} toolicon;

typedef struct toolwin {
  int x0, y0, x1, y1, h;
  canvas c;
  toolicon * i;
  int flags;
  struct toolwin * next;
  void * parent;
  int coord[1];
#ifdef HAVE_BACKGROUND
  void * sprb_area;
  char * sprb_name;
  int sprb_w, sprb_h;
#endif
  tool_redrawp pre_redraw, post_redraw;
} toolwin;

typedef struct toolbar {
  canvas c;
  int x0, y0, x1, y1; /* Window's bounding box */
  int old_w, old_h; /* Old width and height */
  int la, ra, ta, ba; /* L / R / T / B add value to size */
  int contexts, context;
  int above; /* This is the window immediately above it */
  int modecount;
  tool_resize reproc;
  tool_closep closeproc;
  toolwin * w;
  struct toolbar * next;
} toolbar;

toolbar * bars = NULL;

static int tooli_x, tooli_y;
static int last_shift;

void toolbar_fullopen(toolbar * b);

static void __toolbar_geticonsize(void)
{
  if(draw_shifty == 2) {
    tooli_y = 44;
    tooli_x = 42;
  } else {
    tooli_y = 38;
    tooli_x = 38;
  }
}

static int ABS(int n)
{
  if(n < 0) return - n;
  return n;
}

static void __tool_redraw(canvas c, int x0, int y0, int x1, int y1)
{
  toolicon * i;
  toolwin * w = (toolwin *)canvas_idhandle(c);
  int j = 0;
  icon_data d;

#ifdef HAVE_BACKGROUND
  if((w -> sprb_area) && (w -> sprb_w) && (w -> sprb_h)) {
    int xx0, xx1, yy0, yy1;
    xx0 = x0 / w -> sprb_w - 1;
    xx1 = x1 / w -> sprb_w + 1;
    yy0 = y0 / w -> sprb_h - 1;
    yy1 = y1 / w -> sprb_h + 1;
    // swerr(TRUE, "Redrawing from [%d,%d]->[%d,%d] ([%d,%d]->[%d,%d])", x0, y0, x1, y1, xx0, yy0, xx1, yy1);
    d.indirecttext.valid = (char *)w -> sprb_area;
    d.indirecttext.text = (char *)w -> sprb_name;
    d.indirecttext.length = 12;
    for(; yy0 < yy1; yy0 ++) {
      for(j = xx0; j < xx1; j ++) {
	draw_ploticon(j * w -> sprb_w, yy0 * w -> sprb_h, (j + 1) * w -> sprb_w, (yy0 + 1) * w -> sprb_h, 0x102, & d);
      }
    }
  }
#endif

  if(w -> pre_redraw) {
    w -> pre_redraw(w, x0, y0, x1, y1);
  }

  if(draw_scaley == 4) j = 2;
  i = w -> i;
  d.indirecttext.length = 0;
  while(i) {
    if(icon_selected(i -> ic)) {
      d.indirecttext.text = (char *)i -> spr[j + 1];
    } else {
      d.indirecttext.text = (char *)i -> spr[j];
    }
    d.indirecttext.valid = (char *)i -> spra;
    if(icon_faded(i -> ic)) {
      draw_ploticon(i -> x, i -> y, i -> x + 56, i -> y + 56, 0x1740011a, & d);
    } else {
      draw_ploticon(i -> x, i -> y, i -> x + 56, i -> y + 56, 0x1700011a, & d);
    }
    i = i -> next;
  }
  if(w -> post_redraw) {
    w -> post_redraw(w, x0, y0, x1, y1);
  }
}

/* static void __toolbar_void(void)
   {
   } */

void toolbar_close(toolbar * b)
{
  toolwin * w = b -> w;

  while(w) {
    canvas_close(w -> c);
    w = w -> next;
  }
  canvas_close(b -> c);
  if(b -> closeproc) {
    b -> closeproc((void *)b);
  }
}

static void __toolbar_calculatewidths(toolbar * a)
{
  int l = 0, r = 0, t = 0, b = 0;
  toolwin * w = a -> w;

  while(w) {
    if((w -> flags & (1 << 31)) == 0) {
      switch(w -> flags & 3) {
      case TOOL_TOP : t += ABS(w -> h) + draw_scaley; break;
      case TOOL_RIGHT : r += ABS(w -> h) + draw_scalex; break;
      case TOOL_BOTTOM : b += ABS(w -> h) + draw_scaley; break;
      case TOOL_LEFT : l += ABS(w -> h) + draw_scalex; break;
      }
    }
    /* a -> above = canvas_handle(w -> c); */
    w = w -> next;
  }
  a -> la = l;
  a -> ra = r;
  a -> ta = t;
  a -> ba = b;
  canvas_setfullsize(a -> c, a -> x0 - l, a -> y0 - b, a -> x1 + r, a -> y1 + t);
}

static void __toolbar_calculateusage(toolbar * b, int i, int h)
{
  int x = 0, y = 0;
  toolwin * w = b -> w;

  __toolbar_geticonsize();

  b -> old_w = i;
  b -> old_h = h;
  while(w) {
    if((w -> flags & (1 << 31)) == 0) {
      switch(w -> flags & 3) {
      case TOOL_TOP : w -> x0 = x; w -> x1 = i; w -> y0 = h - ABS(w -> h); w -> y1 = h; h -= ABS(w -> h); if(w -> flags & TOOL_HSCROLL) h -= tooli_x; break;
      case TOOL_RIGHT : w -> x0 = i - ABS(w -> h); w -> x1 = i; w -> y0 = y; w -> y1 = h; i -= ABS(w -> h); if(w -> flags & TOOL_VSCROLL) i -= tooli_x; break;
      case TOOL_BOTTOM : w -> x0 = x; w -> x1 = i; w -> y0 = y; w -> y1 = y + ABS(w -> h); y += ABS(w -> h); if(w -> flags & TOOL_HSCROLL) h -= tooli_x; break;
      case TOOL_LEFT : w -> x0 = x; w -> x1 = x + ABS(w -> h); w -> y0 = y; w -> y1 = h; x += ABS(w -> h); if(w -> flags & TOOL_VSCROLL) i -= tooli_x; break;
      }
      if(w -> flags & TOOL_VSCROLL) w -> x1 -= tooli_x;
      if(w -> flags & TOOL_HSCROLL) w -> y0 += tooli_y;
    }
    w = w -> next;
  }
}

static void __toolbar_open(toolbar * b, canvas_openstr * o)
{
  int behind = o -> behind;
  canvas_openstr o2, o3;
  toolwin * t = b -> w;
  int w, h;
  int dp;

  __toolbar_geticonsize();

  if(behind == -3) {
    while(t) {
      canvas_close(t -> c);
      t = t -> next;
    }
    canvas_open(b -> c, o);
    return;
  }

  if(o -> scx < - b -> la + b -> x0) o -> scx = -b -> la + b -> x0;
  /* if(o -> scy > b -> ta - b -> y1) o -> scy = b -> ta - b -> y1; */
  /* if(o -> scx + b -> la + b -> x1 - b -> x0 + b -> ra > b -> ra) o -> scx = b - */

  if(behind == b -> above) {
    dp = 0;
  } else {
    dp = 1;
  }

  canvas_getcanvasstate(b -> c, & o2);

  if(o2.flags & (1 << 21)) {
    if(o2.flags & (1 << 16)) {
      o -> behind = o2.behind;
      canvas_open(b -> c, o);
      canvas_getcanvasstate(b -> c, o);
    }
  }

  w = o -> x1 - o -> x0;
  h = o -> y1 - o -> y0;
  if((w != b -> old_w) || (h != b -> old_h) || (last_shift != draw_shifty)) {
    last_shift = draw_shifty;
    if(w < b -> la + b -> ra) {
      o -> x1 = o -> x0 + b -> la + b -> ra;
      w = o -> x1 - o -> x0;
    }
    if(h < b -> ta + b -> ba) {
      o -> y0 = o -> y1 - (b -> ta + b -> ba);
      h = o -> y1 - o -> y0;
    }
    __toolbar_calculateusage(b, w, h);
    if(b -> reproc) {
      b -> reproc((void *)b, w, h);
    }  } else {
      if(behind == b -> above) return;
    }

  while(t) {
    if((t -> flags & (1 << 31)) == 0) {
      if(!dp) {
	canvas_getcanvasstate(t -> c, & o2);
	memcpy(& o3, & o2, sizeof(o3));
      } else {
	canvas_getcanvasstate(t -> c, & o3);
	o2.behind = behind;
      }
      o2.x0 = o -> x0 + t -> x0;
      o2.y0 = o -> y0 + t -> y0;
      o2.x1 = o -> x0 + t -> x1;
      o2.y1 = o -> y0 + t -> y1;
      if((t -> flags & TOOL_RIGHT) == 0) {
	if(t -> flags & TOOL_SCROLL) o2.scx = o2.x0 - o -> x0 + o -> scx; else o2.scx = 0;
	if(t -> flags & TOOL_CONTEXT) o2.scy = ABS(t -> h) * (b -> context + 1); else {if(t -> h < 0) o2.scy = 0; else o2.scy = o2.y1 - o2.y0; }    } else {
	  if(t -> flags & TOOL_SCROLL) o2.scy = o2.y1 - o -> y1 + o -> scy; else o2.scy = 0;
	  if(t -> flags & TOOL_CONTEXT) o2.scx = ABS(t -> h) * (b -> context); else {if(t -> h < 0) o2.scx = o2.x1 - o2.x0; else o2.scx = 0; }
	}
      if(t -> flags & TOOL_HSCROLL) o2.scx = o3.scx;
      if(t -> flags & TOOL_VSCROLL) o2.scy = o3.scy;
      canvas_open(t -> c, & o2);
      if(dp) {
	behind = canvas_handle(t -> c);
      }   } else {
	canvas_close(t -> c);
      }
    t = t -> next;
  }
  if(dp) {
    o -> behind = behind;
  } else {
    o -> behind = b -> above;
  }
  canvas_open(b -> c, o);
}

void toolbar_wimpopen(toolbar * b, canvas_openstr * o)
{
  __toolbar_open(b, o);
}

void toolbar_scrollopen(toolbar * b, canvas_openstr * o)
{
  int scx, scy;
  toolwin * t = b -> w;
  canvas_openstr o2;

  canvas_open(b -> c, o);
  canvas_getcanvasstate(b -> c, o);
  scx = o -> scx;
  scy = o -> scy;
  while(t) {
    if(((t -> flags & (1 << 31)) == 0) && (t -> flags & TOOL_SCROLL)) {
      canvas_getcanvasstate(t -> c, & o2);
      if((t -> flags & 1) == 0) {
	o2.scx = o2.x0 - o -> x0 + scx;
      } else {
	o2.scy = o2.y1 - o -> y1 + scy;
      }
      canvas_open(t -> c, & o2);
    }
    t = t -> next;
  }
}

static void __toolbar_copen(canvas c, canvas_openstr * o)
{
  toolbar * b = bars;

  while(b) {
    if(b -> c == c) {
      __toolbar_open(b, o);
      return;
    }
    b = b -> next;
  }
}

static void __toolbar_cclose(canvas c)
{
  toolbar * b = bars;

  while(b) {
    if(b -> c == c) {
      toolbar_close(b);
      return;
    }
    b = b -> next;
  }
}

static void __toolbar_cscroll(canvas c, canvas_openstr * o, int xd, int yd)
{
  toolbar * b = bars;
  int w;

  while(b) {
    if(b -> c == c) {
      switch(xd) {
      case CANVAS_SCROLLPAGELEFT : w = (o -> x1 - o -> x0) - (b -> la + b -> ra); if(w > 32) o -> scx -= w - 32; else o -> scx -= 32; break;
      case CANVAS_SCROLLPAGERIGHT : w = (o -> x1 - o -> x0) - (b -> la + b -> ra); if(w > 32) o -> scx += w - 32; else o -> scx += 32; break;
      case CANVAS_SCROLLLEFT : o -> scx -= 32; break;
      case CANVAS_SCROLLRIGHT : o -> scx += 32; break;
      }
      switch(yd) {
      case CANVAS_SCROLLPAGEUP : w = (o -> y1 - o -> y0) - (b -> ta + b -> ba); if(w > 32) o -> scy += w - 32; else o -> scy += 32; break;
      case CANVAS_SCROLLPAGEDOWN : w = (o -> y1 - o -> y0) - (b -> ta + b -> ba); if(w > 32) o -> scy -= w - 32; else o -> scy -= 32; break;
      case CANVAS_SCROLLUP : o -> scy += 32; break;
      case CANVAS_SCROLLDOWN : o -> scy -= 32; break;
      }
      toolbar_scrollopen(b, o);
      return;
    }
    b = b -> next;
  }
}

void toolbar_pagescroll(toolbar * t, int xd, int yd)
{
  canvas_openstr o;
  int w;

  canvas_getcanvasstate(t -> c, & o);

  switch(xd) {
  case CANVAS_SCROLLPAGELEFT : w = (o.x1 - o.x0) - (t -> la + t -> ra); if(w > 32) o.scx -= w - 32; else o.scx -= 32; break;
  case CANVAS_SCROLLPAGERIGHT : w = (o.x1 - o.x0) - (t -> la + t -> ra); if(w > 32) o.scx += w - 32; else o.scx += 32; break;
  case CANVAS_SCROLLLEFT : o.scx -= 32; break;
  case CANVAS_SCROLLRIGHT : o.scx += 32; break;
  }
  switch(yd) {
  case CANVAS_SCROLLPAGEUP : w = (o.y1 - o.y0) - (t -> ta + t -> ba); if(w > 32) o.scy += w - 32; else o.scy += 32; break;
  case CANVAS_SCROLLPAGEDOWN : w = (o.y1 - o.y0) - (t -> ta + t -> ba); if(w > 32) o.scy -= w - 32; else o.scy -= 32; break;
  case CANVAS_SCROLLUP : o.scy += 32; break;
  case CANVAS_SCROLLDOWN : o.scy -= 32; break;
  }
  toolbar_scrollopen(t, & o);
}

toolbar * toolbar_create(int x0, int y0, int x1, int y1, int ctx, char * name)
{
  toolbar * b;

  if(-- ctx < 0) ctx = 0;

  b = rmalloc(sizeof(toolbar));
  if(b == 0) return 0;
  memset(b, 0, sizeof(toolbar));
  b -> c = canvas_create(640, 480, name);
  canvas_setfullsize(b -> c, x0, y0, x1, y1);
  canvas_colour(b -> c, 0);
  canvas_addhandler(b -> c, CANVAS_OPENHANDLER, __toolbar_copen);
  canvas_addhandler(b -> c, CANVAS_CLOSEHANDLER, __toolbar_cclose);
  canvas_addhandler(b -> c, CANVAS_SCROLLHANDLER, __toolbar_cscroll);
  canvas_setflags(b -> c, CANVAS_SCROLLREQUEST, CANVAS_SCROLLREQUEST);
  b -> la = b -> ra = b -> ta = b -> ba = 0;
  b -> old_w = b -> old_h = 65536;
  b -> w = 0;
  b -> next = bars;
  b -> context = 0;
  b -> contexts = ctx;
  bars = b;
  b -> x0 = x0; b -> y0 = y0; b -> y1 = y1; b -> x1 = x1;
  return b;
}

void toolbar_removehscroll(toolbar * b)
{
  canvas_setflags(b -> c, CANVAS_HSCROLL, 0);
}

void toolbar_removevscroll(toolbar * b)
{
  canvas_setflags(b -> c, CANVAS_VSCROLL, 0);
}

void toolbar_addhandler(toolbar * b, int handler, void * proc)
{
  switch(handler) {
  case 0 :
    b -> reproc = (tool_resize)proc;
    break;
  case 1 :
    b -> closeproc = (tool_closep)proc;
  }
}

void toolbar_make(toolbar * b)
{
  canvas_make(b -> c);
}

void toolbar_makeatlocation(toolbar * b, int x, int y)
{
  canvas_makeatlocation(b -> c, x, y);
}

void toolbar_makeatlocationsize(toolbar * b, int x, int y, int w, int h)
{
  canvas_makeatlocationsize(b -> c, x, y, w, h);
}

void toolbar_winmake(toolwin * b)
{
  // canvas_addhandler(b -> c, CANVAS_OPENHANDLER, __toolbar_void);
  canvas_setflags(b -> c, CANVAS_CONTROLSFLAGS | CANVAS_PANE | CANVAS_GENERATEHOTKEYS, CANVAS_NEWFLAGS | CANVAS_PANE);
  if(b -> flags & TOOL_VSCROLL) canvas_setflags(b -> c, 0, CANVAS_VSCROLL);
  if(b -> flags & TOOL_HSCROLL) canvas_setflags(b -> c, 0, CANVAS_HSCROLL);
  canvas_make(b -> c);
}

void toolbar_reopen(toolbar * b)
{
  canvas_openstr o;

  canvas_getcanvasstate(b -> c, & o);
  o.behind = -1;
  __toolbar_open(b, & o);
}

void toolbar_opencentre(toolbar * b)
{
  canvas_openstr o;

  canvas_opencentre(b -> c);
  canvas_getcanvasstate(b -> c, & o);
  o.scx = -b -> la;
  o.scy = b -> ta;
  o.behind = -1;
  b -> old_w = b -> old_h = -1;
  __toolbar_open(b, & o);
}

void toolbar_open(toolbar * b, int x, int y)
{
  canvas_openstr o;

  canvas_opentop(b -> c, x, y);
  canvas_getcanvasstate(b -> c, & o);
  o.scx = -b -> la;
  o.scy = b -> ta;
  o.behind = -1;
  b -> old_w = b -> old_h = -1;
  __toolbar_open(b, & o);
}

toolwin * toolbar_window(toolbar * b, int flags, int height)
{
  toolwin * w, * k;
  int ctxs = b -> contexts;
  if((flags & TOOL_CONTEXT) == 0) ctxs = 1;
  w = rmalloc(sizeof(toolwin) + ctxs * sizeof(int));
  if(w == 0) return 0;
  memset(w, 0, sizeof(toolwin) + ctxs * sizeof(int));
  w -> parent = (void *)b;
  if(b -> w) {
    k = b -> w;
    while(k -> next) k = k -> next;
    k -> next = w;
  } else {
    b -> w = w;
  }
  if(height == 0) height = 72;
  w -> h = height;
  w -> flags = flags;
  w -> next = 0;
  w -> i = 0;
#ifdef HAVE_BACKGROUND
  w -> sprb_area = NULL;
  w -> sprb_name = NULL;
#endif
  w -> pre_redraw = NULL;
  w -> post_redraw = NULL;
  w -> c = canvas_create(32767, 32767, "");
  canvas_setfullsize(w -> c, - 32767, - 32767, 32767, 32767);
  memset(& w -> coord, 0, ctxs * 4 + 4);
  __toolbar_calculatewidths(b);
  canvas_addhandler(w -> c, CANVAS_REDRAWHANDLER, __tool_redraw);
  canvas_setidhandle(w -> c, (int)w);
  return w;
}

#ifdef HAVE_BACKGROUND
void toolbar_winbackground(toolwin * w, void * area, char * name)
{
  int mode, mask;

  w -> sprb_area = area;
  w -> sprb_name = name;

  draw_getnamedspriteinfo(area, name, & w -> sprb_w, & w -> sprb_h, & mode, & mask);
  w -> sprb_w = w -> sprb_w << draw_getmodevar(mode, 4);
  w -> sprb_h = w -> sprb_h << draw_getmodevar(mode, 5);
  canvas_colour(w -> c, 0xff);
}
#endif

void toolbar_winredraw(toolwin * w, tool_redrawp pre, tool_redrawp post)
{
  w -> pre_redraw = pre;
  w -> post_redraw = post;
}

void toolbar_setsize(toolbar * b, int x0, int y0, int x1, int y1)
{
  canvas_openstr o;
  int reopen_full = 0;
  if(b -> c) {
    canvas_getcanvasstate(b -> c, & o);
    if(o.flags & (1 << 18)) reopen_full = TRUE;
  }
  b -> x0 = x0;
  b -> x1 = x1;
  b -> y0 = y0;
  b -> y1 = y1;
  __toolbar_calculatewidths(b);
  if(reopen_full) {
    toolbar_fullopen(b);
  }
}

canvas toolbar_winhandle(toolwin * w)
{
  return w -> c;
}

canvas toolbar_handle(toolbar * w)
{
  return w -> c;
}

icon toolbar_addicon(toolwin * w, int flg, int ctx, void * sprarea, char * spname)
{
  char sname[12];
  toolicon * i;
  void * sa;
  icon ic;
  icon_data d;

  sa = sprite_address(sprarea, spname);
  if(sa == 0) return 0;
  i = rmalloc(sizeof(toolicon));
  if(i == 0) return 0;
  i -> next = w -> i;
  w -> i = i;
  i -> spr[0] = sa;
  i -> spr[1] = sa;
  i -> spr[2] = sa;
  i -> spr[3] = sa;
  sprintf(sname, "p%s", spname);
  if(sa = sprite_address(sprarea, sname), sa) {
    i -> spr[1] = sa;
    i -> spr[3] = sa;
  }
  sprintf(sname, "%s0", spname);
  if(sa = sprite_address(sprarea, sname), sa) {
    i -> spr[2] = sa;
  }
  sprintf(sname, "p%s0", spname);
  if(sa = sprite_address(sprarea, sname), sa) {
    i -> spr[3] = sa;
  }
  i -> spra = sprarea;
  i -> flags = flg;
  switch(w -> flags & 1) {
  case 0 : /* Vertical (x - increases) */
    i -> x = 8 + w -> coord[ctx];
    i -> y = 8 + w -> h * ctx;
    break;
  case 1 : /* Horizontal (y - increases) */
    i -> y = -(8 + w -> coord[ctx]) - 56;
    i -> x = 8 + w -> h * ctx;
    break;
  }
  if((w -> flags & TOOL_CONTEXT) == 0) {
    i -> y = w -> h - 8 - 56;
  }
  d.indirecttext.text = "";
  if(flg & TOOL_TOGGLE) {
    d.indirecttext.valid = (char *) - 1;
  } else {
    d.indirecttext.valid = "r5";
  }
  d.indirecttext.length = 1;
  ic = icon_create(w -> c, 8 + w -> coord[ctx], i -> y, 8 + w -> coord[ctx] + 56, i -> y + 56, 0x3181, & d);
  /* i -> ca = w -> c; */
  i -> ic = ic;
  w -> coord[ctx] += 64;
  return ic;
}

void toolbar_leavegap(toolwin * w, int ctx, int wi)
{
  w -> coord[ctx] += wi + 8;
}

void toolbar_winvisible(toolwin * w, int g)
{
  canvas_openstr o;
  toolbar * b = (toolbar *)w -> parent;
  if(g) {
    w -> flags &= 0x7fffffff;
  } else {
    w -> flags |= 1 << 31;
  }

  __toolbar_calculatewidths(b);
  canvas_getcanvasstate(b -> c, & o);
  /* __toolbar_calculateusage(b, o.x1 - o.x0, o.y1 - o.y0); */
  b -> old_w = -1;
  b -> old_h = -1;
  /* b -> above = -1; */
  if(o.flags & (1 << 16)) {
    __toolbar_open(b, & o);
  }
}

void toolbar_setcontext(toolbar * b, int ctx)
{
  toolwin * t = b -> w;
  canvas_openstr o;

  b -> context = ctx;
  canvas_getcanvasstate(b -> c, & o);
  if(o.flags & CANVAS_OPENED) {
    while(t) {
      if(((t -> flags & (1 << 31)) == 0) && (t -> flags & TOOL_CONTEXT)) {
	canvas_getcanvasstate(t -> c, & o);
	if((t -> flags & 1) == 0) {
	  o.scy = t -> h * (b -> context + 1);
	} else {
	  o.scx = t -> h * (b -> context);
	}
	canvas_open(t -> c, & o);
      }
      t = t -> next;
    }
  }
}

void toolbar_fullopen(toolbar * b)
{
  canvas_openstr o;

  canvas_getcanvasstate(b -> c, & o);
  if(o.flags & CANVAS_OPENEDFULLSIZE) return;

  o.x1 = o.x0 + (b -> x1 - b -> x0) + b -> la + b -> ra;

  if(o.x1 > draw_maxx - 42) {
    o.x0 += (draw_maxx - 42) - o.x1;
    o.x1 = draw_maxx - 42;
  }

  if(o.x0 < 0) o.x0 = 0;
  o.y0 = o.y1 - ((b -> y1 - b -> y0) + b -> ta + b -> ba);

  if(o.y0 < 40) {
    o.y1 -= (o.y0 - 40);
    o.y0 = 40;
  }

  if(o.y1 > draw_maxy - (40)) o.y1 = draw_maxy - (40);

  toolbar_wimpopen(b, & o);
  canvas_getcanvasstate(b -> c, & o);
  toolbar_wimpopen(b, & o);
}

static void toolwin_remove(toolwin * w)
{
  toolicon * ic = w -> i;
  while(ic) {
    toolicon * in = ic -> next;
    icon_remove(ic -> ic);
    free(ic);
    ic = in;
  }
  canvas_remove(w -> c);
  rfree(w);
}

void toolbar_remove(toolbar * b)
{
  toolbar * cptr = bars;
  toolbar * prev = NULL;

  while((cptr != b) && (cptr != NULL)) {
    prev = cptr;
    cptr = cptr -> next;
  }
  if(cptr == NULL) {
    return;
  }

  if(prev == NULL) {
    bars = b -> next;
  }   else {
    cptr -> next = b -> next;
  }

  canvas_remove(b -> c); {
    toolwin * tw = b -> w;

    while(tw) {
      toolwin * tn = tw -> next;
      toolwin_remove(tw);
      tw = tn;
    }
  }
  rfree(b);
}
